require 'gosu'

class Game < Gosu::Window
  def self.play!
    new.show
  end
  
  attr_accessor :input_manager #TODO: make static on InputManager class
  attr_accessor :paused 
  attr_reader :game_state, :world, :player, :enemies, :level, :animation_manager
  def initialize
    super 896, 690, false
    
    $game = self

    @input_manager = InputManager
    @animation_manager = AnimationManager.new(self)
    @game_state = GameState::TitleScreen.new(self)

    self.caption = "Strangeness"
    
    @ui = UI.new(self)
    
    @levels = YAML.load_file(File.join(DATA_ROOT,'levels.yml'))

    MediaManager.play_music
  end
  
  def start_game
    @level = 0
    setup_level
  end

  def draw
    if @game_state.draw_world?
      @ui.draw
      
      Gosu.translate(0,UI_HEIGHT) do
        Gosu.scale(SCALE_FACTOR,SCALE_FACTOR,0,0) do
          MediaManager.image("background").draw(0,0,0)
          Gosu.translate(TILE_WIDTH,TILE_HEIGHT) do
            @world.grid.each(&:draw)
            @game_state.draw unless @game_state.fullscreen?
          end
        end
        MediaManager.image("background0").draw(0,0,0) if @level == 0
      end
      @game_state.draw if @game_state.fullscreen?
    else
      @game_state.draw
    end
  end

  def button_down(id)
    binding.pry if id == Gosu::KB_BACKTICK
    if @game_state.needs_raw_input?
      @game_state.button_down(id)
    else
      @input_manager.button_down(id)
    end
  end
  
  def animation_duration
    @game_state.animation_duration
  end
  
  def handle_input(action)
    case action
    when :quit then exit
    when :toggle_music then MediaManager.toggle_music
    when :toggle_sfx then MediaManager.toggle_sfx
    else @game_state.handle_input(action)
    end
  end
  
  def pause
    @paused = true
    @game_state = GameState::PauseMenu.new
  end
  
  def unpause
    @paused = false
    @game_state = GameState::WaitingForPlayer.new
  end
  
  def paused?
    @paused
  end
  
  def game_state=(state)
    return if @game_state&.locked?
    @game_state.on_exit if @game_state
    @game_state = state
    @game_state.on_enter if @game_state
  end
  
  def update
    @animation_manager.play_if_pending!
    @game_state.update
  end
  
  def player_died
    restart_level
  end
  
  def enemy_died(who)
    enemies.delete(who)
  end
  
  def schedule_animation(anim,&callback)
    @animation_manager.schedule_animation(anim,&callback)
  end
  
  def restart_level
    setup_level
  end
  
  def waiting_allowed?
    false # @world.waiting_allowed?
  end
  
  def next_level
    MediaManager.play_sfx('portal')
    @level += 1
    @level < @levels.count ? setup_level : to_be_continued
  end
  
  def setup_level
    MediaManager.play_song_for_level(@level)
    @world = World.new(@levels[@level])
    #self.game_state = GameState::WaitingForPlayer.new(self)
    self.game_state = GameState::LevelSplashScreen.new(self) { self.game_state = GameState::WaitingForPlayer.new(self) }
  end
  
  def to_be_continued
    self.game_state = GameState::YouWinScreen.new(self)
  end
    
  def player
    @world.player
  end
  
  def enemies
    @world.enemies
  end
  
  def needs_cursor?
    false
  end
  
  def in_menu?
    @game_state.kind_of?(GameState::Menu)
  end
end
